#include "common.h"
#include "irda.h"
#include "blue.h"
#include "pocketmon.h"

#define DEBUG

int main(int argc, char **argv)
{
	int ret;
	short ts_buf[4];
	int selected_pocketmon;
	int attach_direction;

	struct irda_instance irda;
	struct blue_packet bt;

	pthread_t ts_thread;
	pthread_t blue_thread;
	void *thread_result;

	fb_fd=open(FB_DEV, O_RDWR);
	if( fb_fd < 0 )
    	{
		printf("%s open failed\n", FB_DEV);
		exit(-1);
    	}

    	ts_fd=open(TS_DEV, O_RDWR);
    	if( ts_fd < 0 )
    	{
		printf("%s open failed\n", TS_DEV);
		exit(-1);
    	}
	
	sound_fd=open(SOUND_DEV, O_WRONLY);
    	if( sound_fd < 0 )
    	{
		printf("%s open failed\n", SOUND_DEV);
		exit(1);
    	}
	
	init_irda(&irda);

	bt_init();

	pfbdata=(unsigned short *) mmap(0, FB_SIZE*2, PROT_READ | PROT_WRITE, MAP_SHARED, fb_fd, 0);
	
	signal(SIGINT, cleanup);

	init_fb();

	selected_pocketmon=select_pocketmon();

	init_fb();

	init_loading_image();

	pthread_create(&load_thread, NULL, load_thread_func, NULL);

	init_pocketmon_app(selected_pocketmon);

	pthread_join(load_thread, &thread_result);

	init_fb();

	display_pocketmon_image();

	pthread_mutex_init(&mutex, NULL);

	pthread_create(&ts_thread, NULL, ts_thread_func, NULL);

	pthread_create(&blue_thread, NULL, blue_thread_func, &bt);

	while(1)
	{
		if(p_mon->fight_flag == NO_FIGHT)
		{
			if(is_attached(&irda, DOWN_CHECK) || is_attached(&irda, UP_CHECK))
			{
				if(is_attached(&irda, DOWN_CHECK))
					attach_direction=DOWN_ATTACH;
				else if(is_attached(&irda,UP_CHECK))
					attach_direction=UP_ATTACH;

				display_image_to_fb(vs, VS_IMAGE_X_SIZE, VS_IMAGE_Y_SIZE,
						VS_IMAGE_X_OFFSET, VS_IMAGE_Y_OFFSET);
				
				bt.ctl=BT_CTL_POCKETMON_VS;
				bt_send(&bt, p_blue_data, sizeof(p_blue_packet));
#ifdef DEBUG
				printf("Send : bt.ctl=[%d]", bt.ctl); 
				printf("p_blue_data[id:%d]", p_blue_data->id);
				printf("[race:%d]", p_blue_data->race);
				printf("[attack_damage:%d]\n", p_blue_data->attack_damage); 
				sleep(SLEEP_TIME);
#endif

				if(p_mon->fight_flag == FIGHTING)
				{
					init_fb();
					display_pocketmon_image();
				}
					
				while(p_mon->fight_flag ==  FIGHTING)
				{
					switch(attach_direction)
					{
						case DOWN_ATTACH:

							if(!is_attached(&irda, DOWN_CHECK))
							{
								pthread_mutex_lock(&mutex);
								p_mon->fight_flag=NO_FIGHT;
								pthread_mutex_unlock(&mutex);
								bt.ctl=BT_CTL_POCKETMON_NOVS;
								bt_send(&bt, p_blue_data, sizeof(p_blue_packet));
#ifdef DEBUG
								printf("Send : bt.ctl=[%d]", bt.ctl); 
								printf("p_blue_data[id:%d]", p_blue_data->id);
								printf("[race:%d]", p_blue_data->race);
								printf("[attack_damage:%d]\n",p_blue_data->attack_damage);
								sleep(SLEEP_TIME);
								printf("No Fight!\n");
#endif
							}
							break;

						case UP_ATTACH:

							if(!is_attached(&irda, UP_CHECK))
							{
								pthread_mutex_lock(&mutex);
								p_mon->fight_flag=NO_FIGHT;
								pthread_mutex_unlock(&mutex);
								bt.ctl=BT_CTL_POCKETMON_NOVS;
								bt_send(&bt, p_blue_data, sizeof(p_blue_packet));
#ifdef DEBUG
								printf("Send : bt.ctl=[%d]", bt.ctl); 
								printf("p_blue_data[id:%d]", p_blue_data->id);
								printf("[race:%d]", p_blue_data->race);
								printf("[attack_damage:%d]\n",p_blue_data->attack_damage);
								sleep(SLEEP_TIME);
								printf("No Fight!\n");
#endif
							}
							break;
					}
				}
			}
		}

		if(p_mon->life <= 0)
		{			
			game_over();
			bt.ctl=BT_CTL_POCKETMON_END;
			bt_send(&bt, NULL, 0);
#ifdef DEBUF
			printf("Send : bt.ctl=[%d]\n", bt.ctl); 
#endif
			pthread_mutex_lock(&mutex);
			power_off=1;
			pthread_mutex_unlock(&mutex);
		}
		
		if(power_off)
		{
			irda_close(&irda);

			pthread_cancel(ts_thread);
			pthread_join(ts_thread, &thread_result);

			pthread_cancel(blue_thread);
			pthread_join(blue_thread, &thread_result);

			cleanup();
		}
	}
 
	return 0;
}

/********************************************************************************************************
 *
 *  < void select_pocketmon(void) >
 *
 *  -  포켓몬 선택을 위한 인터페이스 함수
 *
 *******************************************************************************************************/  
int select_pocketmon(void)
{
	int fd;
	short ts_buf[4];
	int pocketmon;
	unsigned short pocketmon_title[POCKETMON_TITLE_IMAGE_SIZE];	

	fd=open_image(POCKETMON_TITLE_IMAGE_PATH);
	copy_image_to_buff(fd, pocketmon_title, POCKETMON_TITLE_IMAGE_SIZE);
	close(fd);

	display_image_to_fb(pocketmon_title, POCKETMON_TITLE_IMAGE_X_SIZE, POCKETMON_TITLE_IMAGE_Y_SIZE,
			POCKETMON_TITLE_IMAGE_X_OFFSET, POCKETMON_TITLE_IMAGE_Y_OFFSET);

	while(1)
	{
		if( read(ts_fd, ts_buf, sizeof(ts_buf)) < 0 )
		{
			printf("Read Touch Screen Data Error\n");
			cleanup();
		}
#ifdef DEBUG
		printf("Press[%d] X[%d] Y[%d] MillSec[%d]\n", ts_buf[0], ts_buf[1], ts_buf[2], ts_buf[3]);
#endif
		pocketmon=check_pocketmon_zone(ts_buf[1], ts_buf[2]);
		
		if( ts_buf[0] && ( pocketmon == POCKETMON_PICACHU || pocketmon == POCKETMON_FIRY) )
			break;
	}

	return pocketmon;
}

/**********************************************************************************************************
 *
 *  < void init_pocketmon_app(void) >
 *
 *  - 포켓몬 프로그램에 필요한 이미지, 사운드를 메모리에 로드한다.
 *
 **********************************************************************************************************/
void init_pocketmon_app(int pocketmon_race)
{
	int fd;

	p_mon=(struct pocketmon *)malloc(sizeof(struct pocketmon));
	
	p_mon->race=pocketmon_race;
	p_mon->life=3;
	p_mon->level=1;
	p_mon->exp=0;
	p_mon->fight_flag=NO_FIGHT;
	
	p_mon->p_skill=(struct skill *)malloc(sizeof(struct skill));
	p_mon->p_image=(struct image *)malloc(sizeof(struct image));
	p_mon->p_sound=(struct sound *)malloc(sizeof(struct sound));

	init_pocketmon_skill();

	if(p_mon->race == POCKETMON_PICACHU)
	{
		p_mon->p_sound->pocketmon_attack_wav=(char *)malloc(PICA_ATTACK_WAV_SIZE);
		p_mon->p_sound->wav_size=PICA_ATTACK_WAV_SIZE;
	}
	else
	{
		p_mon->p_sound->pocketmon_attack_wav=(char *)malloc(FIRY_ATTACK_WAV_SIZE);
		p_mon->p_sound->wav_size=FIRY_ATTACK_WAV_SIZE;
	}
	
	p_blue_data=(p_blue_packet *)malloc(sizeof(p_blue_packet));
	p_blue_data->id=MODULE_ID;
	p_blue_data->race=p_mon->race;
	p_blue_data->attack_damage=head_skill->damage;

	switch(p_mon->race)
	{

		case POCKETMON_PICACHU:

			fd=open_image(PICACHU_IMAGE_PATH);
			copy_image_to_buff(fd, p_mon->p_image->pocketmon, POCKETMON_IMAGE_SIZE);
			close(fd);
			
			fd=open_image(PICA_DAMAGED_IMAGE_PATH);
			copy_image_to_buff(fd, p_mon->p_image->damaged_pocketmon, POCKETMON_IMAGE_SIZE);
			close(fd);
			
			fd=open_image(PICA_ATTACK_IMAGE1_PATH);
			copy_image_to_buff(fd, p_mon->p_image->attack1, POCKETMON_IMAGE_SIZE);
			close(fd);

			fd=open_image(PICA_ATTACK_IMAGE2_PATH);
			copy_image_to_buff(fd, p_mon->p_image->attack2, POCKETMON_IMAGE_SIZE);
			close(fd);

			fd=open_image(PICA_ATTACK_IMAGE3_PATH);
			copy_image_to_buff(fd, p_mon->p_image->attack3, POCKETMON_IMAGE_SIZE);
			close(fd);

			fd=open_image(PICA_ATTACK_IMAGE4_PATH);
			copy_image_to_buff(fd, p_mon->p_image->attack4, POCKETMON_IMAGE_SIZE);
			close(fd);

			fd=open_image(PICA_ATTACK_IMAGE5_PATH);
			copy_image_to_buff(fd, p_mon->p_image->attack5, POCKETMON_IMAGE_SIZE);
			close(fd);

			fd=open_image(PICA_ATTACK_IMAGE6_PATH);
			copy_image_to_buff(fd, p_mon->p_image->attack6, POCKETMON_IMAGE_SIZE);
			close(fd);

			fd=open_image(PICA_ATTACK_IMAGE7_PATH);
			copy_image_to_buff(fd, p_mon->p_image->attack7, POCKETMON_IMAGE_SIZE);
			close(fd);

			fd=open_image(PICA_STATE_BAR_IMAGE1_PATH);
			copy_image_to_buff(fd, p_mon->p_image->state_bar[0], STATE_BAR_IMAGE_SIZE);
			close(fd);

			fd=open_image(PICA_STATE_BAR_IMAGE2_PATH);
			copy_image_to_buff(fd, p_mon->p_image->state_bar[1], STATE_BAR_IMAGE_SIZE);
			close(fd);

			fd=open_image(PICA_STATE_BAR_IMAGE3_PATH);
			copy_image_to_buff(fd, p_mon->p_image->state_bar[2], STATE_BAR_IMAGE_SIZE);
			close(fd);

			fd=open_image(PICA_STATE_BAR_IMAGE4_PATH);
			copy_image_to_buff(fd, p_mon->p_image->state_bar[3], STATE_BAR_IMAGE_SIZE);
			close(fd);

			fd=open_wav(PICA_ATTACK_WAV_PATH);
			copy_wav_to_buff(fd, p_mon->p_sound->pocketmon_attack_wav, PICA_ATTACK_WAV_SIZE);
			close(fd);

			break;

		case POCKETMON_FIRY:
	
			fd=open_image(FIRY_IMAGE_PATH);
			copy_image_to_buff(fd, p_mon->p_image->pocketmon, POCKETMON_IMAGE_SIZE);
			close(fd);
			
			fd=open_image(FIRY_IMAGE_PATH);
			copy_image_to_buff(fd, p_mon->p_image->damaged_pocketmon, POCKETMON_IMAGE_SIZE);
			close(fd);
			
			fd=open_image(FIRY_ATTACK_IMAGE1_PATH);
			copy_image_to_buff(fd, p_mon->p_image->attack1, POCKETMON_IMAGE_SIZE);
			close(fd);

			fd=open_image(FIRY_ATTACK_IMAGE2_PATH);
			copy_image_to_buff(fd, p_mon->p_image->attack2, POCKETMON_IMAGE_SIZE);
			close(fd);

			fd=open_image(FIRY_ATTACK_IMAGE3_PATH);
			copy_image_to_buff(fd, p_mon->p_image->attack3, POCKETMON_IMAGE_SIZE);
			close(fd);

			fd=open_image(FIRY_ATTACK_IMAGE4_PATH);
			copy_image_to_buff(fd, p_mon->p_image->attack4, POCKETMON_IMAGE_SIZE);
			close(fd);

			fd=open_image(FIRY_ATTACK_IMAGE5_PATH);
			copy_image_to_buff(fd, p_mon->p_image->attack5, POCKETMON_IMAGE_SIZE);
			close(fd);

			fd=open_image(FIRY_ATTACK_IMAGE6_PATH);
			copy_image_to_buff(fd, p_mon->p_image->attack6, POCKETMON_IMAGE_SIZE);
			close(fd);

			fd=open_image(FIRY_ATTACK_IMAGE6_PATH);
			copy_image_to_buff(fd, p_mon->p_image->attack7, POCKETMON_IMAGE_SIZE);
			close(fd);

			fd=open_image(FIRY_STATE_BAR_IMAGE1_PATH);
			copy_image_to_buff(fd, p_mon->p_image->state_bar[0], STATE_BAR_IMAGE_SIZE);
			close(fd);

			fd=open_image(FIRY_STATE_BAR_IMAGE2_PATH);
			copy_image_to_buff(fd, p_mon->p_image->state_bar[1], STATE_BAR_IMAGE_SIZE);
			close(fd);

			fd=open_image(FIRY_STATE_BAR_IMAGE3_PATH);
			copy_image_to_buff(fd, p_mon->p_image->state_bar[2], STATE_BAR_IMAGE_SIZE);
			close(fd);

			fd=open_image(FIRY_STATE_BAR_IMAGE4_PATH);
			copy_image_to_buff(fd, p_mon->p_image->state_bar[3], STATE_BAR_IMAGE_SIZE);
			close(fd);

			fd=open_wav(FIRY_ATTACK_WAV_PATH);
			copy_wav_to_buff(fd, p_mon->p_sound->pocketmon_attack_wav, FIRY_ATTACK_WAV_SIZE);
			close(fd);

			break;
	}
	
	fd=open_image(POWER_IMAGE_PATH);
	copy_image_to_buff(fd, power, POWER_IMAGE_SIZE);
	close(fd);
	
	fd=open_image(POWER_H_IMAGE_PATH);
	copy_image_to_buff(fd, power_h, POWER_IMAGE_SIZE);
	close(fd);
	
	fd=open_image(INFO_IMAGE_PATH);
	copy_image_to_buff(fd, info, INFO_IMAGE_SIZE);
	close(fd);

	fd=open_image(INFO_H_IMAGE_PATH);
	copy_image_to_buff(fd, info_h, INFO_IMAGE_SIZE);
	close(fd);

	fd=open_image(STOP_IMAGE_PATH);
	copy_image_to_buff(fd, stop, STOP_IMAGE_SIZE);
	close(fd);

	fd=open_image(STOP_H_IMAGE_PATH);
	copy_image_to_buff(fd, stop_h, STOP_IMAGE_SIZE);
	close(fd);

	fd=open_image(ATTACK_UP_IMAGE_PATH);
	copy_image_to_buff(fd, attack_up, ATTACK_IMAGE_SIZE);
	close(fd);

	fd=open_image(ATTACK_DOWN_IMAGE_PATH);
	copy_image_to_buff(fd, attack_down, ATTACK_IMAGE_SIZE);
	close(fd);

	fd=open_image(VS_IMAGE_PATH);
	copy_image_to_buff(fd, vs, VS_IMAGE_SIZE);
	close(fd);
	
	fd=open_image(GAME_OVER_IMAGE_PATH);
	copy_image_to_buff(fd, _game_over, GAME_OVER_IMAGE_SIZE);
	close(fd);

	fd=open_wav(POWER_WAV_PATH);
	copy_wav_to_buff(fd, power_wav, POWER_WAV_SIZE);
	close(fd);
	
	fd=open_wav(INFO_WAV_PATH);
	copy_wav_to_buff(fd, info_wav, INFO_WAV_SIZE);
	close(fd);

	fd=open_wav(CLICK_WAV_PATH);
	copy_wav_to_buff(fd, click_wav, CLICK_WAV_SIZE);
	close(fd);

	load_complete_flag=1;
}

void init_pocketmon_skill(void)
{
	if(p_mon->race == POCKETMON_PICACHU)
	{
		p_mon->p_skill->skill_name=POCKETMON_SKILL_THUNDER;
		p_mon->p_skill->damage=THUNDER_BASIC_DAMAGE;
		p_mon->p_skill->next=NULL;
		head_skill=p_mon->p_skill;
	}else
	{
		p_mon->p_skill->skill_name=POCKETMON_SKILL_FIRE;
		p_mon->p_skill->damage=FIRE_BASIC_DAMAGE;
		p_mon->p_skill->next=NULL;
		head_skill=p_mon->p_skill;
	}

}

/*********************************************************************************************
 *
 *  < void display_pocketmon_image(void) >
 *
 *  - 포켓몬  프로그램과 관련된 초기 이미지 파일을 화면에 보여준다
 *
 *********************************************************************************************/
void display_pocketmon_image(void)
{
	display_image_to_fb(power, POWER_IMAGE_X_SIZE, POWER_IMAGE_Y_SIZE,
			POWER_X_OFFSET, POWER_Y_OFFSET);
			
	display_image_to_fb(p_mon->p_image->pocketmon, POCKETMON_IMAGE_X_SIZE, POCKETMON_IMAGE_Y_SIZE,
			POCKETMON_IMAGE_X_OFFSET, POCKETMON_IMAGE_Y_OFFSET);

	display_image_to_fb(p_mon->p_image->state_bar[0], STATE_BAR_IMAGE_X_SIZE, STATE_BAR_IMAGE_Y_SIZE,
			STATE_BAR_IMAGE_X_OFFSET, STATE_BAR_IMAGE_Y_OFFSET);

	display_image_to_fb(info, INFO_IMAGE_X_SIZE, INFO_IMAGE_Y_SIZE,
			INFO_IMAGE_X_OFFSET, INFO_IMAGE_Y_OFFSET);
	
	display_image_to_fb(attack_up, ATTACK_IMAGE_X_SIZE, ATTACK_IMAGE_Y_SIZE,
			ATTACK_UP_IMAGE_X_OFFSET, ATTACK_UP_IMAGE_Y_OFFSET);

	display_image_to_fb(attack_down, ATTACK_IMAGE_X_SIZE, ATTACK_IMAGE_Y_SIZE,
			ATTACK_DOWN_IMAGE_X_OFFSET, ATTACK_DOWN_IMAGE_Y_OFFSET);

}

/************************************************************************************************************
 *
 *  < void *ts_thread_func(void *arg) >
 *
 *  -  터치 스크린 인터페이스 함수
 *  	POWER_OFF	: 프로그램 종료
 *  	INFO		: 도움말 표시
 *
 ************************************************************************************************************/
void *ts_thread_func(void *arg)
{
	int ret;
	short ts_buf[4];
	
	while(1)
	{
		if(read(ts_fd,ts_buf,sizeof(ts_buf)) < 0)
		{
	    		printf("Read Touch Screen Data Error\n");
	    		cleanup();
		}	
#ifdef DEBUG		
		printf("Press[%d] X[%d] Y[%d] MillSec[%d]\n", ts_buf[0], ts_buf[1], ts_buf[2], ts_buf[3]);
#endif
		ret=check_click_zone(ts_buf[1], ts_buf[2]);
	
		if( ts_buf[0] )
		{
			
			switch( ret )
			{
				case POWER_OFF:

					display_image_to_fb(power_h, POWER_IMAGE_X_SIZE, POWER_IMAGE_Y_SIZE,
						POWER_X_OFFSET, POWER_Y_OFFSET);
				
					play_wav(power_wav, POWER_WAV_SIZE);

					display_image_to_fb(power, POWER_IMAGE_X_SIZE, POWER_IMAGE_Y_SIZE,
						POWER_X_OFFSET, POWER_Y_OFFSET);

					pthread_mutex_lock(&mutex);
					power_off=1;
					pthread_mutex_unlock(&mutex);

				case INFO:

					pthread_mutex_lock(&mutex);
					acc_key=0;
					pthread_mutex_unlock(&mutex);
					
					play_wav(info_wav, INFO_WAV_SIZE);

					pthread_mutex_lock(&mutex);
					acc_key=1;
					pthread_mutex_unlock(&mutex);
					
					break;

				default:

					break;
			}
		}
			
		usleep(100000);
	}
}

void *blue_thread_func(void *arg)
{
	struct blue_packet *bt=(struct blue_packet *) arg;
	struct blue_packet t_bt; 
	struct pocketmon_blue_packet t_blue_data;

	memset(&t_bt, 0, sizeof(struct blue_packet));
	memset(&t_blue_data, 0, sizeof(p_blue_packet));

	memcpy(&t_blue_data, p_blue_data, sizeof(p_blue_packet));

	bt_make_packet(&t_bt, BT_MASTER, BT_SLAVE, BT_CTL_POCKETMON);
	t_bt.ctl=BT_CTL_POCKETMON_START;
	bt_send(&t_bt, &t_blue_data, sizeof(p_blue_packet));
#ifdef DEBUG
	printf("Send : t_bt.ctl=[%d]", t_bt.ctl); 
	printf("t_blue_data[id:%d]", t_blue_data.id);
	printf("[race:%d]", t_blue_data.race);
	printf("[attack_damage:%d]\n", t_blue_data.attack_damage); 
	sleep(SLEEP_TIME);
#endif
	
	while(1)
	{
		
		bt_recv(&t_bt, &t_blue_data, sizeof(p_blue_packet));
#ifdef DEBUF
		printf("Recv : t_bt.ctl=[%d]", t_bt.ctl); 
		printf("t_blue_data[id:%d]", t_blue_data.id);
		printf("[race:%d]", t_blue_data.race);
		printf("[attack_damage:%d]\n", t_blue_data.attack_damage); 
#endif
		if(t_bt.ctl == BT_CTL_POCKETMON_FIGHT)
		{
			pthread_mutex_lock(&mutex);
			p_mon->fight_flag=FIGHTING;
			pthread_mutex_unlock(&mutex);
		}

		if(t_bt.ctl == BT_CTL_POCKETMON_NOVS)
		{
			pthread_mutex_lock(&mutex);
			p_mon->fight_flag=NO_FIGHT;
			pthread_mutex_unlock(&mutex);
		}

		if(t_bt.ctl == BT_CTL_POCKETMON_ATTACK)
			attack();

		if(t_bt.ctl == BT_CTL_POCKETMON_ATTACKED)
			damaged(&t_blue_data);
		
#ifdef DEBUG
		printf("blue thread running\n");
		sleep(1);
#endif
	}
}

/***********************************************************************************
 *
 *  < void *attack_wav_thread_func(void *arg) >
 *
 *  - 공격 효과음을 위한 함수
 *
 ***********************************************************************************/
void *attack_wav_thread_func(void *arg)
{
	play_wav(p_mon->p_sound->pocketmon_attack_wav, p_mon->p_sound->wav_size);
	pthread_exit(0);
}

/*********************************************************************************************************
 *
 *  < int check_click_zone(short x, short, y) >
 *
 *  - 터치 스크린 디바이스로 부터 얻어온 x, y좌표 값을 받아 설정해 놓은 구역에 합당하면
 *    해당 리턴값을 반환한다
 *
 *********************************************************************************************************/    
int check_click_zone(short x, short y)
{
	if( x > 560 && x < 630 )
	{
		if( y > 16 && y < 86 )
			return POWER_OFF;
	}

	if( x > 16 && x < 86 )
	{
		if( y > 16 && y < 86 )
			return INFO;
	}

	if( x > 460 && x < 500 )
	{
		if( y > 320 && y < 360)
			return STOP;
	}

}

/*****************************************************************************************************
 *
 *  < int check_pocketmon_zone(short x, short y) >
 *
 *  - 포켓몬 선택을 위한 터치 스크린 구역을 인식해 주는 함수
 *
 ****************************************************************************************************/
int check_pocketmon_zone(short x, short y)
{
	if( x > 80 && x < 320 )
	{
		if( y > 170 && y < 430 )
			return POCKETMON_PICACHU;
	}

	if( x > 330 && x < 570 )
	{
		if( y > 170 && y < 430 )
			return 	POCKETMON_FIRY;
	}

	return 0;
}

/*******************************************************************************************************
 *
 *  < void attack(void) >
 *
 *  - 공격 함수
 *
 ******************************************************************************************************/
void attack(void)
{
	pthread_t attack_wav_thread;
	void *thread_result;

	pthread_create(&attack_wav_thread, NULL, attack_wav_thread_func, NULL);

	display_attack_effect();

	pthread_join(attack_wav_thread, &thread_result);
}

/*******************************************************************************************************
 *
 *  < void damaged(void) >
 *
 *  - 데미지 함수
 *
 *******************************************************************************************************/

void damaged(struct pocketmon_blue_packet *attacked)
{
	display_damaged_effect();
	
	pthread_mutex_lock(&mutex);
	p_mon->life=p_mon->life - attacked->attack_damage;
	pthread_mutex_unlock(&mutex);

	display_image_to_fb(p_mon->p_image->state_bar[3-p_mon->life], STATE_BAR_IMAGE_X_SIZE, STATE_BAR_IMAGE_Y_SIZE,
			STATE_BAR_IMAGE_X_OFFSET, STATE_BAR_IMAGE_Y_OFFSET);
}


/*******************************************************************************************************
 *
 *  < void display_attack_effect(void)
 *
 *  - 공격에 대한 그래픽적 효과를 보여주는 함수
 *
 *******************************************************************************************************/
void display_attack_effect(void)
{

	display_image_to_fb(p_mon->p_image->attack1, POCKETMON_IMAGE_X_SIZE, POCKETMON_IMAGE_Y_SIZE,
			POCKETMON_IMAGE_X_OFFSET, POCKETMON_IMAGE_Y_OFFSET);

	usleep(100000);

	display_image_to_fb(p_mon->p_image->attack2, POCKETMON_IMAGE_X_SIZE, POCKETMON_IMAGE_Y_SIZE,
			POCKETMON_IMAGE_X_OFFSET, POCKETMON_IMAGE_Y_OFFSET);

	usleep(100000);

	display_image_to_fb(p_mon->p_image->attack3, POCKETMON_IMAGE_X_SIZE, POCKETMON_IMAGE_Y_SIZE,
			POCKETMON_IMAGE_X_OFFSET, POCKETMON_IMAGE_Y_OFFSET);

	usleep(50000);

	display_image_to_fb(p_mon->p_image->attack4, POCKETMON_IMAGE_X_SIZE, POCKETMON_IMAGE_Y_SIZE,
			POCKETMON_IMAGE_X_OFFSET, POCKETMON_IMAGE_Y_OFFSET);

	usleep(50000);

	display_image_to_fb(p_mon->p_image->attack5, POCKETMON_IMAGE_X_SIZE, POCKETMON_IMAGE_Y_SIZE,
			POCKETMON_IMAGE_X_OFFSET, POCKETMON_IMAGE_Y_OFFSET);

	usleep(50000);

	display_image_to_fb(p_mon->p_image->attack6, POCKETMON_IMAGE_X_SIZE, POCKETMON_IMAGE_Y_SIZE,
			POCKETMON_IMAGE_X_OFFSET, POCKETMON_IMAGE_Y_OFFSET);

	usleep(50000);

	display_image_to_fb(p_mon->p_image->attack7, POCKETMON_IMAGE_X_SIZE, POCKETMON_IMAGE_Y_SIZE,
			POCKETMON_IMAGE_X_OFFSET, POCKETMON_IMAGE_Y_OFFSET);

	usleep(50000);
	
	display_image_to_fb(p_mon->p_image->attack6, POCKETMON_IMAGE_X_SIZE, POCKETMON_IMAGE_Y_SIZE,
			POCKETMON_IMAGE_X_OFFSET, POCKETMON_IMAGE_Y_OFFSET);

	usleep(50000);

	display_image_to_fb(p_mon->p_image->attack7, POCKETMON_IMAGE_X_SIZE, POCKETMON_IMAGE_Y_SIZE,
			POCKETMON_IMAGE_X_OFFSET, POCKETMON_IMAGE_Y_OFFSET);

	sleep(1);

	display_image_to_fb(p_mon->p_image->pocketmon, POCKETMON_IMAGE_X_SIZE, POCKETMON_IMAGE_Y_SIZE,
			POCKETMON_IMAGE_X_OFFSET, POCKETMON_IMAGE_Y_OFFSET);
}

/*******************************************************************************************************
 *
 *  < void display_damaged_effect(void)
 *
 *  - 데미지에 대한 그래픽적 효과를 보여주는 함수
 *
 *******************************************************************************************************/

void display_damaged_effect(void)
{
	display_image_to_fb(p_mon->p_image->damaged_pocketmon, POCKETMON_IMAGE_X_SIZE, POCKETMON_IMAGE_Y_SIZE,
			POCKETMON_IMAGE_X_OFFSET, POCKETMON_IMAGE_Y_OFFSET);

	sleep(1);

	display_image_to_fb(p_mon->p_image->pocketmon, POCKETMON_IMAGE_X_SIZE, POCKETMON_IMAGE_Y_SIZE,
			POCKETMON_IMAGE_X_OFFSET, POCKETMON_IMAGE_Y_OFFSET);
}



/***********************************************************************************************
 *
 *  < void game_over(void) >
 *
 *  - 라이프를 다 소진하여 게임오버 처리를 해주는 함수
 *
 **********************************************************************************************/
void game_over(void)
{
	display_image_to_fb(_game_over, GAME_OVER_IMAGE_X_SIZE, GAME_OVER_IMAGE_Y_SIZE,
			GAME_OVER_IMAGE_X_OFFSET, GAME_OVER_IMAGE_Y_OFFSET);
#ifdef DEBUG
	printf("Game Over!!\n");
#endif
	sleep(2);
}

/**********************************************************************************************************
 * 
 * < void cleanup(void) >
 *
 * - 포켓몬 프로그램과 관련된 자원 해제
 *
 **********************************************************************************************************/
void cleanup(void)
{
	init_fb();

	munmap(pfbdata, FB_SIZE*2);

	close(fb_fd);
	close(ts_fd);
	close(sound_fd);

	pthread_mutex_destroy(&mutex);

	free(p_mon);
	free(p_blue_data);

	exit(1);
}
