#include "common.h"
#include "adc.h"
#include "shock.h"

/******************************************************************************************************
 *
 * < main 함수 >
 *
 * 1. 프레임버퍼, 터치스크린, 사운드 장치 드라이버 오픈
 * 2. 프로그램 초기화, 프레임버퍼 초기화
 * 3. 가속도 감지를 위한 acc_thread 생성
 * 4. 터치 스크린을 통한 인터페이스
 * 	- POWER_OFF	: 프로그램 종료
 * 	- INFO		: 도움말 출력
 *
 *******************************************************************************************************/
int main(int argc, char **argv)
{
	int ret;
	short ts_buf[4];

	pthread_t acc_thread;
	void *thread_result;

	fb_fd=open(FB_DEV, O_RDWR);
	if( fb_fd < 0 )
    	{
		printf("%s open failed\n", FB_DEV);
		exit(-1);
    	}

    	ts_fd=open(TS_DEV, O_RDWR);
    	if( ts_fd < 0 )
    	{
		printf("%s open failed\n", TS_DEV);
		exit(-1);
    	}
    	
	sound_fd=open(SOUND_DEV, O_WRONLY);
    	if( sound_fd < 0 )
    	{
		printf("%s open failed\n", SOUND_DEV);
		exit(-1);
    	}

	pfbdata=(unsigned short *) mmap(0, FB_SIZE*2, PROT_READ | PROT_WRITE, MAP_SHARED, fb_fd, 0);

	signal(SIGINT, cleanup);

	init_fb();

	init_loading_image();

	pthread_create(&load_thread, NULL, load_thread_func, NULL);

	init_shock_app();

	pthread_join(load_thread, &thread_result);

	display_shock_info_image();

	init_fb();

	display_shock_image();

	pthread_create(&acc_thread, NULL, acc_thread_func, NULL);
    	
	while(1)
	{
		if(read(ts_fd,ts_buf,sizeof(ts_buf)) < 0)
		{
	    		printf("Read Touch Screen Data Error\n");
	    		cleanup();
		}	
		
		printf("Press[%d] X[%d] Y[%d] MillSec[%d]\n", ts_buf[0], ts_buf[1], ts_buf[2], ts_buf[3]);

		ret=check_click_zone(ts_buf[1], ts_buf[2]);
	
		if( ts_buf[0] )
		{
			
			switch( ret )
			{
				case POWER_OFF:

					display_image_to_fb(power_h, POWER_IMAGE_X_SIZE, POWER_IMAGE_Y_SIZE,
						POWER_X_OFFSET, POWER_Y_OFFSET);
				
					play_wav(power_wav, POWER_WAV_SIZE);

					pthread_cancel(acc_thread);
					pthread_join(acc_thread, &thread_result);

					display_image_to_fb(power, POWER_IMAGE_X_SIZE, POWER_IMAGE_Y_SIZE,
						POWER_X_OFFSET, POWER_Y_OFFSET);

					cleanup();

				case INFO:
					
					acc_key=0;

					display_image_to_fb(info_h, INFO_IMAGE_X_SIZE, INFO_IMAGE_Y_SIZE,
							INFO_IMAGE_X_OFFSET, INFO_IMAGE_Y_OFFSET);
					
					play_wav(info_wav, INFO_WAV_SIZE);

					display_shock_info_image();

					init_fb();

					display_shock_image();
					
					acc_key=1;

					break;

				default:

					break;
			}
		}
			
		usleep(100000);
	}

	cleanup();
}

/***********************************************************************************
 *
 * < void init_shock_app(void) >
 *
 * - 쇼크 프로그램에 필요한 이미지 파일과 사운드 파일을 메모리로 로드한다
 *
 ***********************************************************************************/
void init_shock_app(void)
{
	int fd;
	
	fd=open_image(POWER_IMAGE_PATH);
	copy_image_to_buff(fd, power, POWER_IMAGE_SIZE);
	close(fd);
	
	fd=open_image(POWER_H_IMAGE_PATH);
	copy_image_to_buff(fd, power_h, POWER_IMAGE_SIZE);
	close(fd);
	
	fd=open_image(INFO_IMAGE_PATH);
	copy_image_to_buff(fd, info, INFO_IMAGE_SIZE);
	close(fd);

	fd=open_image(INFO_H_IMAGE_PATH);
	copy_image_to_buff(fd, info_h, INFO_IMAGE_SIZE);
	close(fd);

	fd=open_image(STOP_IMAGE_PATH);
	copy_image_to_buff(fd, stop, STOP_IMAGE_SIZE);
	close(fd);

	fd=open_image(STOP_H_IMAGE_PATH);
	copy_image_to_buff(fd, stop_h, STOP_IMAGE_SIZE);
	close(fd);
	
	fd=open_image(SHOCK_MANUAL_IMAGE_PATH);
	copy_image_to_buff(fd, shock_manual, SHOCK_MANUAL_IMAGE_SIZE);
	close(fd);
	
	fd=open_image(SHOCK_IMAGE1_PATH);
	copy_image_to_buff(fd, shock_image1, SHOCK_IMAGE1_SIZE);
	close(fd);

	fd=open_image(SHOCK_IMAGE2_PATH);
	copy_image_to_buff(fd, shock_image2, SHOCK_IMAGE2_SIZE);
	close(fd);
	
	fd=open_wav(POWER_WAV_PATH);
	copy_wav_to_buff(fd, power_wav, POWER_WAV_SIZE);
	close(fd);
	
	fd=open_wav(INFO_WAV_PATH);
	copy_wav_to_buff(fd, info_wav, INFO_WAV_SIZE);
	close(fd);

	fd=open_wav(CLICK_WAV_PATH);
	copy_wav_to_buff(fd, click_wav, CLICK_WAV_SIZE);
	close(fd);
	
	fd=open_wav(BOX_OPEN_WAV_PATH);
	copy_wav_to_buff(fd, box_open_wav, BOX_OPEN_WAV_SIZE);
	close(fd);
	
	load_complete_flag=1;
}

/*********************************************************************************************
 *
 *  < void display_shock_image(void) >
 *
 *  - 쇼크 프로그램과 관련된 초기 이미지 파일을 화면에 보여준다
 *
 *********************************************************************************************/
void display_shock_image(void)
{
	display_image_to_fb(power, POWER_IMAGE_X_SIZE, POWER_IMAGE_Y_SIZE,
			POWER_X_OFFSET, POWER_Y_OFFSET);

	display_image_to_fb(info, INFO_IMAGE_X_SIZE, INFO_IMAGE_Y_SIZE,
			INFO_IMAGE_X_OFFSET, INFO_IMAGE_Y_OFFSET);

	display_image_to_fb(shock_image1, SHOCK_IMAGE1_X_SIZE, SHOCK_IMAGE1_Y_SIZE,
			SHOCK_IMAGE_X_OFFSET, SHOCK_IMAGE_Y_OFFSET);
}

/**********************************************************************************************
 *
 *  < void display_shock_info_image(void) >
 *
 *  - 사용자가 인포 버튼을 눌렀을 경우에 해당 도움말을 보여주고 터치 스크린을 통해
 *    도움말 인터페이스를 컨트롤 한다
 *
 *********************************************************************************************/
void display_shock_info_image(void)
{
	int ret;
	short ts_buf[4];
		
	display_image_to_fb(shock_manual, SHOCK_MANUAL_IMAGE_X_SIZE, SHOCK_MANUAL_IMAGE_Y_SIZE,
				SHOCK_MANUAL_IMAGE_X_OFFSET, SHOCK_MANUAL_IMAGE_Y_OFFSET);
	
	display_image_to_fb(stop, STOP_IMAGE_X_SIZE, STOP_IMAGE_Y_SIZE,
			STOP_IMAGE_X_OFFSET, STOP_IMAGE_Y_OFFSET);

	while(1)
	{
		ret=read(ts_fd, ts_buf, sizeof(ts_buf));
		if( ret < 0 )
		{
	    		printf("Read Touch Screen Data Error\n");
			exit(-1);
		}

		ret=check_click_zone(ts_buf[1], ts_buf[2]);

		if( ts_buf[0] && ret == STOP )
		{

			display_image_to_fb(stop_h, STOP_IMAGE_X_SIZE, STOP_IMAGE_Y_SIZE,
					STOP_IMAGE_X_OFFSET, STOP_IMAGE_Y_OFFSET);
				
			play_wav(click_wav, CLICK_WAV_SIZE);

			break;
		}
	}
}

/********************************************************************************************************
 *
 *  < void *acc_thread_func(void *arg) >
 *
 *  - 사용자가 모듈을 조작함으로써 얻은 가속도 값을 통해 화면의 그림을 바꿔준다.
 *  - check_acc()함수를 통해 사용자가 바닥면에 충격을 주면 흔들린 가속도를 통해 얻은 값이
 *    일정 값에 도달하면 1을 리턴하고 화면의 그림을 변화시킨다.
 *
 ********************************************************************************************************/
void *acc_thread_func(void *arg)
{
	int start_cnt=0;
	struct struct_accel *acc;
	struct struct_accel_data *data;
	
	acc=acc_init(ACC_CAL_X, ACC_CAL_Y, ACC_CAL_Z);
	data=acc->struct_accel_data;

	while(1)
	{
		printf("--------------------------\n%lf %lf %lf\n", data->dxv, data->dyv, data->dzv);
		printf("%d %d %d\n", data->rawx, data->rawy, data->rawz);
		printf("%lf %lf %lf\n", data->xtheta, data->ytheta, data->ztheta);
		printf("%lf %lf %lf\n\n", data->xacc, data->yacc, data->zacc);
		
		if( acc->check_acc(50000, ADC_ACCEL | ACC_SHOCK) && start_cnt == 5 )
		{ 
			if(acc_key)
			{

				display_image_to_fb(shock_image2, SHOCK_IMAGE2_X_SIZE, SHOCK_IMAGE2_Y_SIZE,
						SHOCK_IMAGE_X_OFFSET, SHOCK_IMAGE_Y_OFFSET);

				play_wav(box_open_wav, BOX_OPEN_WAV_SIZE);

				sleep(1);

				display_image_to_fb(shock_image1, SHOCK_IMAGE1_X_SIZE, SHOCK_IMAGE2_Y_SIZE,
						SHOCK_IMAGE_X_OFFSET, SHOCK_IMAGE_Y_OFFSET);

				start_cnt=0;
			}
		
		}

		if(start_cnt < 5)
			start_cnt++;
	}
}

/*********************************************************************************************************
 *
 *  < int check_click_zone(short x, short, y) >
 *
 *  - 터치 스크린 디바이스로 부터 얻어온 x, y좌표 값을 받아 설정해 놓은 구역에 합당하면
 *    해당 리턴값을 반환한다
 *
 *********************************************************************************************************/
int check_click_zone(short x, short y)
{
	if( x > 560 && x < 630 )
	{
		if( y > 16 && y < 86 )
			return POWER_OFF;
	}

	if( x > 16 && x < 86 )
	{
		if( y > 16 && y < 86 )
			return INFO;
	}

	if( x > 460 && x < 500 )
	{
		if( y > 320 && y < 360)
			return STOP;
	}

}

/**********************************************************************************************************
 * 
 * < void cleanup(void) >
 *
 * - 쇼크 프로그램과 관련된 자원 해제
 *
 **********************************************************************************************************/   
void cleanup(void)
{
	init_fb();

	munmap(pfbdata, FB_SIZE*2);

	close(fb_fd);
	close(ts_fd);
	close(sound_fd);

	acc_exit();

	exit(1);
}
