#ifndef ADC_H
#define ADC_H

#define PI		3.14159265358979323846	/* pi */
//#define BPV		0.000805664 /* bit per volt */
#define ACC_BPV		0.0008056640625 /* bit per volt */
#define ACC_SENS		0.8 /* acc sensor sensitivity */

/* radian과 theta 간의 상호변환 매크로 */
#define theta2rad(x) ((double)(x) * (PI/180))
#define rad2theta(x) ((double)(x) * (180/PI))




/* 센서 종류구분과 검출대상 구분 플래그 */

/* ---------------------------------------------------------------------------------------------
			BIT MAP
 31  30  29  28  27  26  25  24  23  22  21  20  19  18  17  16 15 14 13  12 11 10 9  8  7  6  5  4  3  2  1 
|-----------------------------------------------------------|------------|-----------------------|----------|
	               free                                      piezo            acc선택            센서
-------------------------------------------------------------------------------------------------------------
*/

/* 센서 종류 */
#define ADC_ACCEL 0x01
#define ADC_PIEZO 0x02

/* 가속도 행동 구분 */
#define ACC_SHOCK 0x10
#define ACC_DIRECTION 0x20
#define ACC_DROP 0x40
#define ACC_SHAKE 0x80
#define ACC_MENU_DROP 0x100

/* 피에조 행동 구분 */
#define PIEZO_WIND 0x1000

/* 센서종류와 행동검출을위한 마스크비트 정의 */
#define ADC_SEL_MASK 0x0f
#define ACC_SEL_MASK 0xff0
#define PIEZO_SEL_MASK 0xf000

#define __CALLBACK


struct struct_accel_data {
	unsigned short rawx, rawy, rawz;

	/* 센서에서 들어온 디지털값을 디지털Voltage값으로 변환한 X Y Z */
	/* ex) dxv = 센서에서읽은값 * BPV */
	/* "BPV는 Bit Per volt. 센서리턴값 1비트당 볼트값 */
	double dxv, dyv, dzv;

	/* 중력가속도 g를 저장한 변수 */
	/* ex) xacc = (dxv - 1.65 + xcalib) / SENS */
	/* SENS는 가속도센서 하드웨어 민감도 설정값 */
	double xacc, yacc, zacc;

	/* 구해진 중력가속도를 기반으로 실제 각도를 저장 */
	/* ex) ztheta = atan(sqrt(pow(xacc, 2) + pow(yacc, 2)) / zacc) */
	double xtheta, ytheta, ztheta;


};

struct struct_accel {
	/* calibration data. X Y Z 기준전압은 1.65V이고 칼리브레이션 변수와의 합에의해 최종결정 */
	double xcalib, ycalib, zcalib;

	/* adc.c 의 check_adc함수와 연결된다 */
	int (*check_acc)(int, const unsigned int);
	/* adc.c 의 acc_exit함수와 연결된다(소멸자) */
	void (*exit)();

	/* struct_accel_data구조체와 연결 */
	struct struct_accel_data *struct_accel_data;

};

struct struct_piezo {
	unsigned short raw;
	unsigned short lowlimit, highlimit;
	int (*check_piezo)();
	void (*exit)();	
};



/* struct_accel initialize 함수 */
extern struct struct_accel *acc_init(double xcalib, double ycalib, double zcalib);
extern struct struct_piezo *piezo_init(unsigned short lowlimit, unsigned short highlimit);


#endif
